/****************************************************************************
TMDS transmitter PHY for Xilinx 7-series

2021.4 Tatsuyuki Sato

note:
    parameter

    SERIALIZER
        "VIVADO_IP"   : Vivado IP Catalog ("OSERDESE2") , IO_MODE only "TMDS_33" 
        "FAB_DDR"     : generic HDL + "ODDR" , DDR clock
        "FAB_SDR"     : generic HDL , SDR clock (not suported)
    IO_MODE
        "PSEDO_PP"    : psedo PushPull  , 2xSINGLE END (LVCMOS_33) + ext. Register
        "PSEDO_OD"    : psedo OpenDrain , 2xSINGLE END (LVCMOS_33) + ext. Register
        "TMDS_33"     : true TMDS , TMDS_33

****************************************************************************/
module tmds_tx_phy_xc7 #(
    SERIALIZER = "FAB_DDR",      
    IO_MODE    = "PSEDO_PP"
)(
    // input
    input           I_RESET,              //
    input           I_ICLK,               // input capture clock
    input   [9:0]   I_CH0 , I_CH1, I_CH2, // 10bit TMDS data,MSB=1st out,LSB=last out
    input           I_PCLK,               // output period clock , x1 I_ICLK
    input           I_SCLK,               // output serial clock , x5 I_SCLK
    output  [3:0]   O_TMDS_P, O_TMDS_N    // output PADs
);

/****************************************************************************
serializer
****************************************************************************/
wire [3:0] tmds_p,tmds_n;                // internal reg.
generate
if( SERIALIZER == "VIVADO_IP")
begin
    // Vivado IP catalog::SelectIO Interface Wizard::DVI Transmitter
    // output data matrix assign
    wire    [29:0] data_out_from_device =
    {//    CH2          CH1          CH0
        I_CH2[9],I_CH1[9],I_CH0[9], // 1st out
        I_CH2[8],I_CH1[8],I_CH0[8], // 2nd out
        I_CH2[7],I_CH1[7],I_CH0[7],
        I_CH2[6],I_CH1[6],I_CH0[6],
        I_CH2[5],I_CH1[5],I_CH0[5],
        I_CH2[4],I_CH1[4],I_CH0[4],
        I_CH2[3],I_CH1[3],I_CH0[3],
        I_CH2[2],I_CH1[2],I_CH0[2],
        I_CH2[1],I_CH1[1],I_CH0[1],
        I_CH2[0],I_CH1[0],I_CH0[0] // last out
    };
    // IP Catalog module
    TMDS_TX tmds_serialzier
    (
        .io_reset   (I_RESET),
        .clk_reset  (I_RESET),
        .clk_div_in (I_PCLK),
        .clk_in     (I_SCLK),
        .data_out_from_device(data_out_from_device),
        // swap DVI to HDMI channel
        .clk_to_pins_p     (O_TMDS_P[3]),    // TMDS_33 PAD
        .clk_to_pins_n     (O_TMDS_N[3]),    // TMDS_33 PAD
        .data_out_to_pins_p(O_TMDS_P[2:0]),  // TMDS_33 PAD
        .data_out_to_pins_n(O_TMDS_N[2:0])   // TMDS_33 PAD
    );
    assign {tmds_p,tmds_n}='d0;    // dummy
end else begin // if( SERIALIZER == "FAB_DDR")
    // generic HDL serializer
    tmdstx_xc7_oddr tmds_serialzier_oddr (
        .reset(I_RESET),
        .clkin(I_PCLK)   , .clkx5p(I_SCLK)  , .clkx5n(~I_SCLK),
        .data_in0(I_CH0) , .data_in1(I_CH1) , .data_in2(I_CH2),
        .tmds_p(tmds_p)  , .tmds_n(tmds_n)
    );
    // OoutPort Driver
end
endgenerate

/****************************************************************************
I/O DRIVER
****************************************************************************/
generate
if( IO_MODE == "TMDS_33")
begin
    if( SERIALIZER != "VIVADO_IP") // OBUFTDS is included in TMDS_TX

    begin
        // true TMDS
        OBUFTDS TMDS0 (.I(tmds_p[0]), .T(I_RESET), .O(O_TMDS_P[0]), .OB(O_TMDS_N[0])) ;
        OBUFTDS TMDS1 (.I(tmds_p[1]), .T(I_RESET), .O(O_TMDS_P[1]), .OB(O_TMDS_N[1])) ;
        OBUFTDS TMDS2 (.I(tmds_p[2]), .T(I_RESET), .O(O_TMDS_P[2]), .OB(O_TMDS_N[2])) ;
        OBUFTDS TMDSC (.I(tmds_p[3]), .T(I_RESET), .O(O_TMDS_P[3]), .OB(O_TMDS_N[3])) ;
    end
end
if( IO_MODE == "PSEDO_PP")
begin
    // psedo differencial LVCMOS_33-push-pull
    OBUF PSTMDS_P0 (.I(tmds_p[0]), .O(O_TMDS_P[0])) ;
    OBUF PSTMDS_N0 (.I(tmds_n[0]), .O(O_TMDS_N[0])) ;
    OBUF PSTMDS_P1 (.I(tmds_p[1]), .O(O_TMDS_P[1])) ;
    OBUF PSTMDS_N1 (.I(tmds_n[1]), .O(O_TMDS_N[1])) ;
    OBUF PSTMDS_P2 (.I(tmds_p[2]), .O(O_TMDS_P[2])) ;
    OBUF PSTMDS_N2 (.I(tmds_n[2]), .O(O_TMDS_N[2])) ;
    OBUF PSTMDS_PC (.I(tmds_p[3]), .O(O_TMDS_P[3])) ;
    OBUF PSTMDS_NC (.I(tmds_n[3]), .O(O_TMDS_N[3])) ;
end
if( IO_MODE == "PSEDO_OD")
begin
    // psedo differencial  LVCMOS_33-opendorain
    OBUFT PSTMDS_P0 (.I(1'b0),.T( tmds_p[0]), .O(O_TMDS_P[0])) ;
    OBUFT PTMDS0_N0 (.I(1'b0),.T( tmds_n[0]), .O(O_TMDS_N[0])) ;
    OBUFT PSTMDS_P1 (.I(1'b0),.T( tmds_p[1]), .O(O_TMDS_P[1])) ;
    OBUFT PSTMDS_N1 (.I(1'b0),.T( tmds_n[1]), .O(O_TMDS_N[1])) ;
    OBUFT PSTMDS_P2 (.I(1'b0),.T( tmds_p[2]), .O(O_TMDS_P[2])) ;
    OBUFT PSTMDS_N2 (.I(1'b0),.T( tmds_n[2]), .O(O_TMDS_N[2])) ;
    OBUFT PSTMDS_PC (.I(1'b0),.T( tmds_p[3]), .O(O_TMDS_P[3])) ;
    OBUFT PSTMDS_NC (.I(1'b0),.T( tmds_n[3]), .O(O_TMDS_N[3])) ;
end
endgenerate

endmodule
