//******************************************************************//
// VIDEO SIGNAL GENERATER
//******************************************************************//

`timescale 1 ps / 1 ps

module videogen
#(
parameter HPIXELS = 12'd1024, //Horizontal Live Pixels
parameter HFNPRCH = 12'd24,   //Horizontal Front Portch
parameter HSYNCPW = 12'd136,  //HSYNC Pulse Width
parameter HBKPRCH = 12'd160,  //Horizontal Back Portch
parameter HSYNC_POLARITY = 1'b1, //0-Negative, 1-Positive
//
parameter VLINES  = 12'd768,  //Vertical Live ines
parameter VBKPRCH = 12'd29,   //Vertical Back Portch
parameter VSYNCPW = 12'd6,    //VSYNC Pulse Width
parameter VFNPRCH = 12'd3,    //Vertical Front Portch
parameter VSYNC_POLARITY = 1'b1 //0-Negative, 1-Positive
)(
  input  wire I_RESET,
// TEST PATTERN SELECT
  input wire [7:0] TEST_PATTERN, // 0=BLANK , 1=COLOR BAR , 2=GRADARION SCROLL
// VIDEO OUTPUT
  input  wire I_OCLK,
  output wire [7:0] O_R,
  output wire [7:0] O_G,
  output wire [7:0] O_B,
  output wire O_HBLANK,
  output wire O_VBLANK,
  output wire O_HSYNC,
  output wire O_VSYNC
);

///////////////////////////////////////////////////////////////////////////
// Video Timing Parameters
///////////////////////////////////////////////////////////////////////////

wire [11:0] tc_hsblnk;
wire [11:0] tc_hssync;
wire [11:0] tc_hesync;
wire [11:0] tc_heblnk;
wire [11:0] tc_vsblnk;
wire [11:0] tc_vssync;
wire [11:0] tc_vesync;
wire [11:0] tc_veblnk;

// event counts
assign tc_hsblnk = HPIXELS - 12'd1;
assign tc_hssync = HPIXELS - 12'd1 + HFNPRCH;
assign tc_hesync = HPIXELS - 12'd1 + HFNPRCH + HSYNCPW;
assign tc_heblnk = HPIXELS - 12'd1 + HFNPRCH + HSYNCPW + HBKPRCH;
assign tc_vsblnk =  VLINES - 12'd1;
assign tc_vssync =  VLINES - 12'd1 + VFNPRCH;
assign tc_vesync =  VLINES - 12'd1 + VFNPRCH + VSYNCPW;
assign tc_veblnk =  VLINES - 12'd1 + VFNPRCH + VSYNCPW + VBKPRCH;

wire VGA_HSYNC_INT, VGA_VSYNC_INT;
wire   [11:0] bgnd_hcount;
wire          bgnd_hsync;
wire          bgnd_hblnk;
wire   [11:0] bgnd_vcount;
wire          bgnd_vsync;
wire          bgnd_vblnk;

timing timing_inst (
  .tc_hsblnk(tc_hsblnk), //input
  .tc_hssync(tc_hssync), //input
  .tc_hesync(tc_hesync), //input
  .tc_heblnk(tc_heblnk), //input
  .hcount(bgnd_hcount), //output
  .hsync(VGA_HSYNC_INT), //output
  .hblnk(bgnd_hblnk), //output
  .tc_vsblnk(tc_vsblnk), //input
  .tc_vssync(tc_vssync), //input
  .tc_vesync(tc_vesync), //input
  .tc_veblnk(tc_veblnk), //input
  .vcount(bgnd_vcount), //output
  .vsync(VGA_VSYNC_INT), //output
  .vblnk(bgnd_vblnk), //output
  .restart(I_RESET),
  .clk(I_OCLK)
);

///////////////////////////////////
// Video pattern generator:
//   SMPTE HD Color Bar
///////////////////////////////////
wire [7:0] red_data, green_data, blue_data;
wire vblank, hblank, vsync, hsync;

wire [7:0] cbar_r,cbar_g,cbar_b;
hdcolorbar #(
  .HDISP(HPIXELS),
  .VDISP(VLINES)
)colorbar_inst(
  .i_clk_74M(I_OCLK),
  .i_rst(I_RESET),
  .i_hcnt(bgnd_hcount),
  .i_vcnt(bgnd_vcount),
  .baronly(1'b0),
  .i_format(2'b00),
  .o_r(cbar_r),
  .o_g(cbar_g),
  .o_b(cbar_b)
);

wire [7:0] gra_r,gra_g,gra_b;
video_gradation gradation_inst(
  .i_reset(I_RESET),
  .i_clk(I_OCLK),
  .i_hblank(hblank),
  .i_vblank(vblank),
  .i_vcnt(bgnd_vcount),
  .i_hcnt(bgnd_hcount),
  .o_red(gra_r),
  .o_green(gra_g),
  .o_blue(gra_b)
);

// sync video pattern generator pipeline
reg [1:0] hsync_d , vsync_d, hblnk_d ,vblnk_d;
always @ (posedge I_OCLK)
begin
  hsync_d <= {VGA_HSYNC_INT ^ ~HSYNC_POLARITY,hsync_d[1]};
  vsync_d <= {VGA_VSYNC_INT ^ ~VSYNC_POLARITY,vsync_d[1]};
  hblnk_d <= {bgnd_hblnk,hblnk_d[1]};
  vblnk_d <= {bgnd_vblnk,vblnk_d[1]};
end
assign {hsync,vsync,hblank,vblank} = {hsync_d[0],vsync_d[0],hblnk_d[0],vblnk_d[0]};

// test pattern selector
assign {red_data,green_data,blue_data} =
  (TEST_PATTERN==8'h01) ? {cbar_r,cbar_g,cbar_b} : // static COLOR BAR
  (TEST_PATTERN==8'h02) ? {gra_r,gra_g,gra_b} :   // dynamic scrolled gradation 
  'd0; // BLANK

///////////////////////////////////
// output 
///////////////////////////////////
assign O_R = red_data;
assign O_G = green_data;
assign O_B = blue_data;
assign O_HBLANK = hblank;
assign O_VBLANK = vblank;
assign O_HSYNC = hsync;
assign O_VSYNC = vsync;

endmodule
