/****************************************************************************
HDMI audio test for ZYBO

2017.9 Tatsuyuki Sato

note:

sw[1] : audio select : off = 1KHz sin wave / 1KHz tone wave
sw[0] : video select : off = basic color bar / on = gradation scrolll 

led[3] : 1Hz base clock indicator
led[2] : HMDI PHY ready (HDMI transmitter enabled)

****************************************************************************/

`timescale 1 ps / 1 ps

/****************************************
 TOP module
****************************************/
module hdmiaudio_zybo #
(
  parameter I2C_DUMMY_READ = 1  
) (
  input  wire clk,      // 125MHz
// HDMI
  output wire hdmi_clk_n,
  output wire hdmi_clk_p,
  output wire [2:0] hdmi_d_p,
  output wire [2:0] hdmi_d_n,
//  output wire hdmi_hpd,
  inout wire hdmi_cec,
  input wire hdmi_hpd,
  output wire hdmi_out_en,
  output wire hdmi_scl,
  inout wire hdmi_sda,
// VGA
   output wire [4:0] vga_r,
   output wire [5:0] vga_g,
   output wire [4:0] vga_b,
   output wire vga_hs,
   output wire vga_vs,
// switch
  input  wire [3:0] sw,
// LED
  output  wire [3:0] led
);
/****************************************
 video mode and clock generator defines
 ***************************************/
// choice video mode define
`include "vga_pll_pll.vh"
//`include "fhd_pll_pll.vh"

/****************************************
 audio formart defines
 ****************************************/
// aiudio parameter
`include "audio_lpcm32k.vh"
//`include "audio_lpcm48k.vh"

//******************************************************************//
// 27MHz base clock Generation
//******************************************************************//
// input buffer
wire clk125m;
IBUFG clk125m_bufg (.I(clk), .O(clk125m));

wire pll27m;
wire pll27m_locked;
pll125m27m pll125m27m(
  .CLK125M_I(clk125m),
  .CLK27M_O(pll27m),
  .reset(1'b0),
  .locked(pll27m_locked)
);

wire clk27m        = pll27m;
wire clk27m_locked = pll27m_locked;

//******************************************************************//
// video pixel clock & I/O clock Generation
//******************************************************************//
//wire pclk_src;
wire pclk_locked;
wire pclk_ready;
wire pclk;
// for OSERDES-DDR mode
wire pclkx5,pclkx1;

// un-used
//wire pclkx2;

//******************************************************************//
// pixes clock generation with 7series MMCM
//******************************************************************//
wire mmcm_pclkx5;
wire mmcm_clkfb;
// MMCM BASED CLOCK
`PCLK_PLL_MODULE PCLK_GEN(    // clk27m25p2m PCLK_GEN(
  .CLK_IN(clk27m),
  .clkfb_in(mmcm_clkfb),
  .clkfb_out(mmcm_clkfb),
//
  .CLK_X5(mmcm_pclkx5),
  .CLK_X5P(),.CLK_X5N(),.CLK_X2(),.CLK_X1(),
  .reset(~clk27m_locked || ~pll27m_locked),
  .locked(pclk_ready)
);
assign pclk_locked =  pclk_ready;

// fast clock for SERDES
BUFIO BUFIO_pclkx5 (
  .O(pclkx5), // 1-bit output: Clock output (connect to I/O clock loads).
  .I(mmcm_pclkx5)  // 1-bit input: Clock input (connect to an IBUF or BUFMR).
);

// slow clock for SERDES , PIXEL 
BUFR #(
   .BUFR_DIVIDE("5"),   // Values: "BYPASS, 1, 2, 3, 4, 5, 6, 7, 8" 
   .SIM_DEVICE("7SERIES")  // Must be set to "7SERIES" 
)
BUFR_pclk (
   .O(pclk),     // 1-bit output: Clock output port
   .CE(pclk_ready),   // 1-bit input: Active high, clock enable (Divided modes only)
   .CLR(0), // 1-bit input: Active high, asynchronous clear (Divided modes only)
   .I(mmcm_pclkx5)      // 1-bit input: Clock buffer input driven by an IBUF, MMCM or local interconnect
);

//******************************************************************//
// video generator
//******************************************************************//
wire [7:0] i_vga_r;
wire [7:0] i_vga_g;
wire [7:0] i_vga_b;
wire vga_hblank;
wire vga_vblank;
wire vga_hsync;
wire vga_vsync;

// video test patern selector
wire [7:0] test_pattern =
     sw[0] ? 8'h02 : // sw[0] = on  :   2=GRADARION SCROLL
             8'h01 ; // sw[0] = off :   1=COLOR BAR

videogen
#(
// video timming
 .HPIXELS(HPIXELS), 
 .HFNPRCH(HFNPRCH), 
 .HSYNCPW(HSYNCPW),
 .HBKPRCH(HBKPRCH),
 .HSYNC_POLARITY(HSYNC_POLARITY),
 .VLINES(VLINES),
 .VBKPRCH(VBKPRCH),
 .VSYNCPW(VSYNCPW),
 .VFNPRCH(VFNPRCH),
 .VSYNC_POLARITY(VSYNC_POLARITY)
)
videogen
(
  .I_RESET(~pclk_ready),
// TEST PATTERN select
  .TEST_PATTERN(test_pattern), // 0=BLANK , 1=COLOR BAR , 2=GRADARION SCROLL
// VIDEO OUTPUT
  .I_OCLK(pclk),
  .O_R(i_vga_r),
  .O_G(i_vga_g),
  .O_B(i_vga_b),
  .O_HBLANK(vga_hblank),
  .O_VBLANK(vga_vblank),
  .O_HSYNC(vga_hsync),
  .O_VSYNC(vga_vsync)
);

//////////////////////////////////////////////////////////////////////
// timming monitor
//////////////////////////////////////////////////////////////////////
reg [25:0] divider1k;
reg clk1khz;

always @(posedge clk27m)
begin
  if (divider1k==(26'd27000/2-1) )
  begin
    divider1k <= 26'd0;
    clk1khz   <= ~clk1khz;
  end else begin
    divider1k <= divider1k + 26'd1;
  end
end

reg [25:0] divider;
reg clk1hz;
always @(posedge clk27m)
begin
  if (divider==(26'd27000000/2-1) )
  begin
    divider <= 26'd0;
    clk1hz  <= ~clk1hz;
  end else begin
    divider <= divider + 26'd1;
  end
end

//////////////////////////////////////////////////////////////////////
// audio signal GENERATOR
//////////////////////////////////////////////////////////////////////
localparam AUDIO_STEP_1KHZ = 40'h0001000000 * 1000 / `DEF_AUDIO_RATE / 128;
wire auidio_128fs;
wire [23:0] audioLch;
wire [23:0] audioRch;

wire [7:0] audioSelect =
     sw[1] ? 8'h02 :    // on  : square wave
             8'h01 ;    // off : sin wave
audiogen audiogen (
 .reset(~pclk_locked),
 .clk(pclk),
 .clken(auidio_128fs),
 .phaseStep24(AUDIO_STEP_1KHZ),
 .selectL(audioSelect),
 .selectR(audioSelect),
 .audioL(audioLch),
 .audioR(audioRch)
);

//////////////////////////////////////////////////////////////////////
// HDMI ENCODER MAC
//////////////////////////////////////////////////////////////////////
wire [9:0] tmds10_ch0,tmds10_ch1,tmds10_ch2;

hdmi_encoder #(
// VIDEO CODE for AVI infoframe packet
  .AVIIF_VIC(`DEF_VIDEO_CODE),
// AUDIO CLOCK INFORMATION
  .TMDS_CLOCK_HZ(`DEF_TMDS_CLOCK),
  .AUDIO_128FS_HZ(`DEF_FS128),
// AUDIO SAMPLING EVENT GENERATION
  .AUDIO_TIMER_ADDITION(`DEF_TIMER_ADD),
  .AUDIO_TIMER_LIMIT(`DEF_TIMER_LIMIT),
// audio clock regeneration verification parameter
  .AUDIO_CLOCK_N(`DEF_ACLOCK_N),
  .AUDIO_CLOCK_CTS(`DEF_ACLOCK_CTS),
// AUDIO frame CHANNEL STATUS (Lch Value)
  .AUDIO_CHS_DATA(`DEF_AUDIO_CHS)
)
hdmi_tx_mac
(
    .rstin      (~pclk_locked),
// video input
    .clkin      (pclk),
    .blue_din   (i_vga_b),
    .green_din  (i_vga_g),
    .red_din    (i_vga_r),
    .hsync      (~vga_hsync),
    .vsync      (~vga_vsync),
    .de         (~vga_hblank && ~vga_vblank),
// audio clock regeneration setting
    .audioclock_n(`DEF_ACLOCK_N),
    .audioclock_cts(`DEF_ACLOCK_CTS),
// audio sampling signal
	.auido_128fs(auidio_128fs),
	.auido_1fs(),
// audio input
    .audio_l(audioLch),
    .audio_r(audioRch),
// HDMI output
    .tmds_data2 (tmds10_ch2),
    .tmds_data1 (tmds10_ch1),
    .tmds_data0 (tmds10_ch0) 
);

////////////////////////////////////////////////////////////////
// HDMI ENCODER PHY
///////////////////////////////////////////////////////////////
wire phy_reset = ~pclk_locked;

// remappin data to IP core
wire    [29:0] data_out_from_device =
{
    tmds10_ch2[9],tmds10_ch1[9],tmds10_ch0[9],
    tmds10_ch2[8],tmds10_ch1[8],tmds10_ch0[8],
    tmds10_ch2[7],tmds10_ch1[7],tmds10_ch0[7],
    tmds10_ch2[6],tmds10_ch1[6],tmds10_ch0[6],
    tmds10_ch2[5],tmds10_ch1[5],tmds10_ch0[5],
    tmds10_ch2[4],tmds10_ch1[4],tmds10_ch0[4],
    tmds10_ch2[3],tmds10_ch1[3],tmds10_ch0[3],
    tmds10_ch2[2],tmds10_ch1[2],tmds10_ch0[2],
    tmds10_ch2[1],tmds10_ch1[1],tmds10_ch0[1],
    tmds10_ch2[0],tmds10_ch1[0],tmds10_ch0[0]
};
TMDS_TX tmds_serialzier // Vivado IP Catalog
(
    .io_reset(phy_reset),
// clock
    .clk_reset(~pclk_locked),
    .clk_div_in(pclk),
    .clk_in(pclkx5),
// input data
    .data_out_from_device(data_out_from_device),
// output pin
    .clk_to_pins_p(hdmi_clk_p),
    .clk_to_pins_n(hdmi_clk_n),
    .data_out_to_pins_p({hdmi_d_p[2],hdmi_d_p[1],hdmi_d_p[0]}),
    .data_out_to_pins_n({hdmi_d_n[2],hdmi_d_n[1],hdmi_d_n[0]})
);
wire phy_ready = ~phy_reset;

///////////////////////////////////////////////////////////////////////
// HDMI DDC I2C dummy access
///////////////////////////////////////////////////////////////////////
reg [15:0] div200k;
reg clk200k;
always @(posedge clk27m)
begin
  if (div200k==(16'd67-1) )
  begin
    div200k <= 16'd0;
    clk200k <= ~clk200k;
  end else begin
    div200k <= div200k + 16'd1;
  end
end

//                    ST  A6   A5   A4   A3   A2   A1   A0   W    A   ED
wire [39:0] scl = 40'b11_0011_0011_0011_0011_0011_0011_0011_0011_0011_11;
wire [39:0] sda = 40'b01_1111_1000_0111_0000_0000_0000_0000_0000_0000_01;

reg [7:0] i2c_phase;
always @(posedge clk200k)
begin
  i2c_phase <= i2c_phase + 1;
  if(i2c_phase==8'd39) i2c_phase <= 8'd0;
end
assign hdmi_scl = I2C_DUMMY_READ ? (scl[i2c_phase] ? 1'b1 : 1'b0) : 1'bz;
assign hdmi_sda = I2C_DUMMY_READ ? (sda[i2c_phase] ? 1'bz : 1'b0) : 1'bz;

///////////////////////////////////////////////////////////////////////
// HDMI direction & CEC-control
///////////////////////////////////////////////////////////////////////
assign hdmi_cec    = 1'bz;
assign hdmi_out_en = 1'b1;

///////////////////////////////////////////////////////////////////////
// Analog VGA output
///////////////////////////////////////////////////////////////////////
wire vga_blank = vga_hblank | vga_vblank;
assign vga_r  = vga_blank ? 5'h00 : i_vga_r[7:3];
assign vga_g  = vga_blank ? 6'h00 : i_vga_g[7:2];
assign vga_b  = vga_blank ? 5'h00 : i_vga_b[7:3];
assign vga_vs = vga_vsync;
assign vga_hs = vga_hsync;

///////////////////////////////////////////////////////////////////////
// LED / SW
///////////////////////////////////////////////////////////////////////
assign led[3] = clk1hz;
assign led[2] = phy_ready;
assign led[1] = sw[1];
assign led[0] = sw[0];

endmodule
