/////////////////////////////////////////////////////////////////////////////
//
// Audio Signal Generator
//
/////////////////////////////////////////////////////////////////////////////
module audiogen
(
 input  reset,
 input  clk,
 input  clken,
 input  [23:0]   phaseStep24,
 input  [7:0]    selectL,     // 00=silent,01=sin,02=square
 input  [7:0]    selectR,     // 00=silent,01=sin,02=square
 output signed [23:0] audioL,
 output signed [23:0] audioR
);
reg  [23:0] audioPhase;
wire signed [13:0] sin14;
wire signed [23:0] sin1khz;
wire signed [23:0] square1khz;

// phase generator
always @(posedge clk)
begin
  if(reset)
  begin
    audioPhase <= 'd0;
  end else if(clken)
  begin
    audioPhase <= audioPhase + phaseStep24;
  end
end

// sin wave geneartor
sinwave_rom sinwave_rom (.C(clk),.CE(clken),.I(audioPhase[23:14]),.O(sin14));
assign sin1khz = ( sin14*(1024/16) ); // -24dB ?

// square wave generator
assign square1khz = audioPhase[23] ? +(24'h7FFFFF/16) : -(24'h7FFFFF/16);

// selecter
audioSelector selL(.SEL(selectL),.I1(sin1khz),.I2(square1khz),.O(audioL));
audioSelector selR(.SEL(selectR),.I1(sin1khz),.I2(square1khz),.O(audioR));

endmodule

/////////////////////////////////////////////////////////////////////////////
// audio selector
/////////////////////////////////////////////////////////////////////////////
module audioSelector
(
  input         [ 7:0]   SEL,
  input  signed [23:0]   I1,
  input  signed [23:0]   I2,
  output signed [23:0]   O
);
assign O = 
 SEL==8'h01 ? I1 :
 SEL==8'h02 ? I2 :
 'd0;
endmodule

/////////////////////////////////////////////////////////////////////////////
// Sinwave ROM
/////////////////////////////////////////////////////////////////////////////

module sinwave_rom(
input  C,
input  CE,
output [13:0] O,
input  [9:0] I
);

// QUARTER ROM
reg [12:0] TBL_SIN;

// HALF TABLE (0-90 degree)
wire [7:0] deg_quater = I[8] ? ~I[7:0] : I[7:0];

// FULL TABLE
assign O = I[9] ? -TBL_SIN : TBL_SIN;

always @(posedge C)
if(CE) begin
case(deg_quater)
 8'h00:TBL_SIN = 13'h000D;  8'h01:TBL_SIN = 13'h003F;
 8'h02:TBL_SIN = 13'h0071;  8'h03:TBL_SIN = 13'h00A3;
 8'h04:TBL_SIN = 13'h00D6;  8'h05:TBL_SIN = 13'h0108;
 8'h06:TBL_SIN = 13'h013A;  8'h07:TBL_SIN = 13'h016D;
 8'h08:TBL_SIN = 13'h019F;  8'h09:TBL_SIN = 13'h01D1;
 8'h0a:TBL_SIN = 13'h0203;  8'h0b:TBL_SIN = 13'h0236;
 8'h0c:TBL_SIN = 13'h0268;  8'h0d:TBL_SIN = 13'h029A;
 8'h0e:TBL_SIN = 13'h02CC;  8'h0f:TBL_SIN = 13'h02FE;
 8'h10:TBL_SIN = 13'h0330;  8'h11:TBL_SIN = 13'h0362;
 8'h12:TBL_SIN = 13'h0394;  8'h13:TBL_SIN = 13'h03C6;
 8'h14:TBL_SIN = 13'h03F8;  8'h15:TBL_SIN = 13'h042A;
 8'h16:TBL_SIN = 13'h045C;  8'h17:TBL_SIN = 13'h048E;
 8'h18:TBL_SIN = 13'h04BF;  8'h19:TBL_SIN = 13'h04F1;
 8'h1a:TBL_SIN = 13'h0523;  8'h1b:TBL_SIN = 13'h0555;
 8'h1c:TBL_SIN = 13'h0586;  8'h1d:TBL_SIN = 13'h05B8;
 8'h1e:TBL_SIN = 13'h05E9;  8'h1f:TBL_SIN = 13'h061B;
 8'h20:TBL_SIN = 13'h064C;  8'h21:TBL_SIN = 13'h067D;
 8'h22:TBL_SIN = 13'h06AE;  8'h23:TBL_SIN = 13'h06E0;
 8'h24:TBL_SIN = 13'h0711;  8'h25:TBL_SIN = 13'h0742;
 8'h26:TBL_SIN = 13'h0773;  8'h27:TBL_SIN = 13'h07A4;
 8'h28:TBL_SIN = 13'h07D4;  8'h29:TBL_SIN = 13'h0805;
 8'h2a:TBL_SIN = 13'h0836;  8'h2b:TBL_SIN = 13'h0866;
 8'h2c:TBL_SIN = 13'h0897;  8'h2d:TBL_SIN = 13'h08C7;
 8'h2e:TBL_SIN = 13'h08F8;  8'h2f:TBL_SIN = 13'h0928;
 8'h30:TBL_SIN = 13'h0958;  8'h31:TBL_SIN = 13'h0988;
 8'h32:TBL_SIN = 13'h09B8;  8'h33:TBL_SIN = 13'h09E8;
 8'h34:TBL_SIN = 13'h0A18;  8'h35:TBL_SIN = 13'h0A47;
 8'h36:TBL_SIN = 13'h0A77;  8'h37:TBL_SIN = 13'h0AA7;
 8'h38:TBL_SIN = 13'h0AD6;  8'h39:TBL_SIN = 13'h0B05;
 8'h3a:TBL_SIN = 13'h0B34;  8'h3b:TBL_SIN = 13'h0B63;
 8'h3c:TBL_SIN = 13'h0B92;  8'h3d:TBL_SIN = 13'h0BC1;
 8'h3e:TBL_SIN = 13'h0BF0;  8'h3f:TBL_SIN = 13'h0C1F;
 8'h40:TBL_SIN = 13'h0C4D;  8'h41:TBL_SIN = 13'h0C7B;
 8'h42:TBL_SIN = 13'h0CAA;  8'h43:TBL_SIN = 13'h0CD8;
 8'h44:TBL_SIN = 13'h0D06;  8'h45:TBL_SIN = 13'h0D34;
 8'h46:TBL_SIN = 13'h0D62;  8'h47:TBL_SIN = 13'h0D8F;
 8'h48:TBL_SIN = 13'h0DBD;  8'h49:TBL_SIN = 13'h0DEA;
 8'h4a:TBL_SIN = 13'h0E17;  8'h4b:TBL_SIN = 13'h0E44;
 8'h4c:TBL_SIN = 13'h0E71;  8'h4d:TBL_SIN = 13'h0E9E;
 8'h4e:TBL_SIN = 13'h0ECB;  8'h4f:TBL_SIN = 13'h0EF7;
 8'h50:TBL_SIN = 13'h0F24;  8'h51:TBL_SIN = 13'h0F50;
 8'h52:TBL_SIN = 13'h0F7C;  8'h53:TBL_SIN = 13'h0FA8;
 8'h54:TBL_SIN = 13'h0FD4;  8'h55:TBL_SIN = 13'h0FFF;
 8'h56:TBL_SIN = 13'h102B;  8'h57:TBL_SIN = 13'h1056;
 8'h58:TBL_SIN = 13'h1082;  8'h59:TBL_SIN = 13'h10AD;
 8'h5a:TBL_SIN = 13'h10D7;  8'h5b:TBL_SIN = 13'h1102;
 8'h5c:TBL_SIN = 13'h112D;  8'h5d:TBL_SIN = 13'h1157;
 8'h5e:TBL_SIN = 13'h1181;  8'h5f:TBL_SIN = 13'h11AB;
 8'h60:TBL_SIN = 13'h11D5;  8'h61:TBL_SIN = 13'h11FF;
 8'h62:TBL_SIN = 13'h1228;  8'h63:TBL_SIN = 13'h1252;
 8'h64:TBL_SIN = 13'h127B;  8'h65:TBL_SIN = 13'h12A4;
 8'h66:TBL_SIN = 13'h12CD;  8'h67:TBL_SIN = 13'h12F5;
 8'h68:TBL_SIN = 13'h131E;  8'h69:TBL_SIN = 13'h1346;
 8'h6a:TBL_SIN = 13'h136E;  8'h6b:TBL_SIN = 13'h1396;
 8'h6c:TBL_SIN = 13'h13BD;  8'h6d:TBL_SIN = 13'h13E5;
 8'h6e:TBL_SIN = 13'h140C;  8'h6f:TBL_SIN = 13'h1433;
 8'h70:TBL_SIN = 13'h145A;  8'h71:TBL_SIN = 13'h1481;
 8'h72:TBL_SIN = 13'h14A8;  8'h73:TBL_SIN = 13'h14CE;
 8'h74:TBL_SIN = 13'h14F4;  8'h75:TBL_SIN = 13'h151A;
 8'h76:TBL_SIN = 13'h1540;  8'h77:TBL_SIN = 13'h1565;
 8'h78:TBL_SIN = 13'h158A;  8'h79:TBL_SIN = 13'h15B0;
 8'h7a:TBL_SIN = 13'h15D4;  8'h7b:TBL_SIN = 13'h15F9;
 8'h7c:TBL_SIN = 13'h161E;  8'h7d:TBL_SIN = 13'h1642;
 8'h7e:TBL_SIN = 13'h1666;  8'h7f:TBL_SIN = 13'h168A;
 8'h80:TBL_SIN = 13'h16AD;  8'h81:TBL_SIN = 13'h16D1;
 8'h82:TBL_SIN = 13'h16F4;  8'h83:TBL_SIN = 13'h1717;
 8'h84:TBL_SIN = 13'h1739;  8'h85:TBL_SIN = 13'h175C;
 8'h86:TBL_SIN = 13'h177E;  8'h87:TBL_SIN = 13'h17A0;
 8'h88:TBL_SIN = 13'h17C2;  8'h89:TBL_SIN = 13'h17E4;
 8'h8a:TBL_SIN = 13'h1805;  8'h8b:TBL_SIN = 13'h1826;
 8'h8c:TBL_SIN = 13'h1847;  8'h8d:TBL_SIN = 13'h1868;
 8'h8e:TBL_SIN = 13'h1888;  8'h8f:TBL_SIN = 13'h18A8;
 8'h90:TBL_SIN = 13'h18C8;  8'h91:TBL_SIN = 13'h18E8;
 8'h92:TBL_SIN = 13'h1907;  8'h93:TBL_SIN = 13'h1927;
 8'h94:TBL_SIN = 13'h1946;  8'h95:TBL_SIN = 13'h1964;
 8'h96:TBL_SIN = 13'h1983;  8'h97:TBL_SIN = 13'h19A1;
 8'h98:TBL_SIN = 13'h19BF;  8'h99:TBL_SIN = 13'h19DD;
 8'h9a:TBL_SIN = 13'h19FA;  8'h9b:TBL_SIN = 13'h1A17;
 8'h9c:TBL_SIN = 13'h1A34;  8'h9d:TBL_SIN = 13'h1A51;
 8'h9e:TBL_SIN = 13'h1A6E;  8'h9f:TBL_SIN = 13'h1A8A;
 8'ha0:TBL_SIN = 13'h1AA6;  8'ha1:TBL_SIN = 13'h1AC2;
 8'ha2:TBL_SIN = 13'h1ADD;  8'ha3:TBL_SIN = 13'h1AF8;
 8'ha4:TBL_SIN = 13'h1B13;  8'ha5:TBL_SIN = 13'h1B2E;
 8'ha6:TBL_SIN = 13'h1B48;  8'ha7:TBL_SIN = 13'h1B62;
 8'ha8:TBL_SIN = 13'h1B7C;  8'ha9:TBL_SIN = 13'h1B96;
 8'haa:TBL_SIN = 13'h1BAF;  8'hab:TBL_SIN = 13'h1BC8;
 8'hac:TBL_SIN = 13'h1BE1;  8'had:TBL_SIN = 13'h1BFA;
 8'hae:TBL_SIN = 13'h1C12;  8'haf:TBL_SIN = 13'h1C2A;
 8'hb0:TBL_SIN = 13'h1C42;  8'hb1:TBL_SIN = 13'h1C59;
 8'hb2:TBL_SIN = 13'h1C70;  8'hb3:TBL_SIN = 13'h1C87;
 8'hb4:TBL_SIN = 13'h1C9E;  8'hb5:TBL_SIN = 13'h1CB4;
 8'hb6:TBL_SIN = 13'h1CCA;  8'hb7:TBL_SIN = 13'h1CE0;
 8'hb8:TBL_SIN = 13'h1CF6;  8'hb9:TBL_SIN = 13'h1D0B;
 8'hba:TBL_SIN = 13'h1D20;  8'hbb:TBL_SIN = 13'h1D35;
 8'hbc:TBL_SIN = 13'h1D49;  8'hbd:TBL_SIN = 13'h1D5D;
 8'hbe:TBL_SIN = 13'h1D71;  8'hbf:TBL_SIN = 13'h1D85;
 8'hc0:TBL_SIN = 13'h1D98;  8'hc1:TBL_SIN = 13'h1DAB;
 8'hc2:TBL_SIN = 13'h1DBE;  8'hc3:TBL_SIN = 13'h1DD0;
 8'hc4:TBL_SIN = 13'h1DE2;  8'hc5:TBL_SIN = 13'h1DF4;
 8'hc6:TBL_SIN = 13'h1E05;  8'hc7:TBL_SIN = 13'h1E17;
 8'hc8:TBL_SIN = 13'h1E28;  8'hc9:TBL_SIN = 13'h1E38;
 8'hca:TBL_SIN = 13'h1E49;  8'hcb:TBL_SIN = 13'h1E59;
 8'hcc:TBL_SIN = 13'h1E69;  8'hcd:TBL_SIN = 13'h1E78;
 8'hce:TBL_SIN = 13'h1E87;  8'hcf:TBL_SIN = 13'h1E96;
 8'hd0:TBL_SIN = 13'h1EA5;  8'hd1:TBL_SIN = 13'h1EB3;
 8'hd2:TBL_SIN = 13'h1EC1;  8'hd3:TBL_SIN = 13'h1ECF;
 8'hd4:TBL_SIN = 13'h1EDC;  8'hd5:TBL_SIN = 13'h1EEA;
 8'hd6:TBL_SIN = 13'h1EF6;  8'hd7:TBL_SIN = 13'h1F03;
 8'hd8:TBL_SIN = 13'h1F0F;  8'hd9:TBL_SIN = 13'h1F1B;
 8'hda:TBL_SIN = 13'h1F27;  8'hdb:TBL_SIN = 13'h1F32;
 8'hdc:TBL_SIN = 13'h1F3D;  8'hdd:TBL_SIN = 13'h1F48;
 8'hde:TBL_SIN = 13'h1F52;  8'hdf:TBL_SIN = 13'h1F5C;
 8'he0:TBL_SIN = 13'h1F66;  8'he1:TBL_SIN = 13'h1F70;
 8'he2:TBL_SIN = 13'h1F79;  8'he3:TBL_SIN = 13'h1F82;
 8'he4:TBL_SIN = 13'h1F8A;  8'he5:TBL_SIN = 13'h1F93;
 8'he6:TBL_SIN = 13'h1F9B;  8'he7:TBL_SIN = 13'h1FA2;
 8'he8:TBL_SIN = 13'h1FAA;  8'he9:TBL_SIN = 13'h1FB1;
 8'hea:TBL_SIN = 13'h1FB8;  8'heb:TBL_SIN = 13'h1FBE;
 8'hec:TBL_SIN = 13'h1FC4;  8'hed:TBL_SIN = 13'h1FCA;
 8'hee:TBL_SIN = 13'h1FD0;  8'hef:TBL_SIN = 13'h1FD5;
 8'hf0:TBL_SIN = 13'h1FDA;  8'hf1:TBL_SIN = 13'h1FDF;
 8'hf2:TBL_SIN = 13'h1FE3;  8'hf3:TBL_SIN = 13'h1FE7;
 8'hf4:TBL_SIN = 13'h1FEB;  8'hf5:TBL_SIN = 13'h1FEE;
 8'hf6:TBL_SIN = 13'h1FF1;  8'hf7:TBL_SIN = 13'h1FF4;
 8'hf8:TBL_SIN = 13'h1FF6;  8'hf9:TBL_SIN = 13'h1FF8;
 8'hfa:TBL_SIN = 13'h1FFA;  8'hfb:TBL_SIN = 13'h1FFC;
 8'hfc:TBL_SIN = 13'h1FFD;  8'hfd:TBL_SIN = 13'h1FFE;
 8'hfe:TBL_SIN = 13'h1FFF;  8'hff:TBL_SIN = 13'h1FFF;
 endcase
end

endmodule
