/****************************************************************************
HDMI encoder MAC with audio data , Tatsuyuki Sato 2016,2017

based reference design : HDMI output for Neo Geo MVS , Charlie Cole 2015

note :

pipeline 

stage 1 :
  capture input data
  sync polarity
stage 2 :
  detect XY-counter and phase
stage 3 :
  8b10b   encode (1/2)
  terc4   data generation
  control data generation
  generate data selector
stage 4 :
  8b10b encode (2/2)
  4b10b encode
  2b10b encode
stage 5
  multiplex 8b10b / 4b10b / 2b10b / guardband

****************************************************************************/

/////////////////////////////////////////////////////////////
// DEBUG CONTROL

// disable data island controll
//`define DISABLE_DATA_ISLAND

// generate check sum error in infoframe packeyt
//`define CAUSE_CHECKSUM_ERROR

// generate ECC calcration error
//`define CAUSE_ECC_ERROR

////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////
// module
module hdmi_encoder
////////////////////////////////////////////////////////////////////////
// parameters
#(
	// AVI INFORMATION
	parameter [6:0] AVIIF_VIC      = 7'h01, // VIC=01 : 640x480p@59.94/60Hz, 4:3,PCLK=25.175MHz / 25.2MHz
	parameter real TMDS_CLOCK_HZ   = 25200000,
	// AUDIO INFORMATION
	parameter real AUDIO_128FS_HZ  = (48000*128),
	// AUDIO CLOCK REGENERATION
	parameter [19:0] AUDIO_CLOCK_N   = (48000*128)/1000, // AUDIO_128FS_HZ / feedback_rate(300 - 1500)
	parameter [19:0] AUDIO_CLOCK_CTS = 25200000/1000,    // TMDS_CLOCK_HZ  / feedback_rate
	// AUDIO capture rate
	parameter AUDIO_TIMER_ADDITION =  4,
	parameter AUDIO_TIMER_LIMIT    = 25200000*4/48000, // AUDIO_CLOCK_CTS*AUDIO_TIMER_ADDITION/(AUDIO_128FS_HZ/128)
	// AUDIO frame CHANNEL STATUS (Lch Value)
	parameter [191:0] AUDIO_CHS_DATA = 192'h0_db_02_01_40_04 // 48KHz 24-bit LPCM audio
)
////////////////////////////////////////////////////////////////////////
// ports
(
	// VIDEO INPUT
	input  wire       clkin,          // pixel clock
	input  wire       rstin,          // reset
	input  wire [7:0] blue_din,       // Blue data in
	input  wire [7:0] green_din,      // Green data in
	input  wire [7:0] red_din,        // Red data in
	input  wire       hsync,          // hsync data
	input  wire       vsync,          // vsync data
	input  wire       de,             // data enable
	// SOUND INPUT
	input  wire [23:0] audio_l,
	input  wire [23:0] audio_r,
	output wire auido_128fs,
	output wire auido_1fs,
	// audio clock regeneration (static or dynamic)
	input wire [19:0] audioclock_n,
	input wire [19:0] audioclock_cts,
	// DHMI OUT
	output wire [9:0] tmds_data2,   // CH2 / R
	output wire [9:0] tmds_data1,   // CH1 / G
	output wire [9:0] tmds_data0    // CH0 / B
);

////////////////////////////////////////////////////////////////////////
// VIDEO TIMMING
////////////////////////////////////////////////////////////////////////

// INFORMATION DATA
localparam [6:0]   audio_regen_interval = AUDIO_CLOCK_N/128;
localparam [191:0] channelStatus        = AUDIO_CHS_DATA;

////////////////////////////////////////////////////////////////////////
// VIDEO TIMMING checker
////////////////////////////////////////////////////////////////////////

// clock parameter verification
localparam real audioclock_cts_calc    = TMDS_CLOCK_HZ*AUDIO_CLOCK_N/AUDIO_128FS_HZ;
localparam real audioclock_128fs_param = TMDS_CLOCK_HZ*AUDIO_CLOCK_N/AUDIO_CLOCK_CTS;
localparam real audioclock_128fs_regen = TMDS_CLOCK_HZ*AUDIO_CLOCK_N/audioclock_cts_calc;

localparam real audioclock_ppm         = (1.0-(audioclock_128fs_param/audioclock_128fs_regen))*1000000000000.0;
localparam real audio_regen_frequency  = (AUDIO_128FS_HZ/AUDIO_CLOCK_N*audio_regen_interval);

initial
begin
	if( ((AUDIO_128FS_HZ/AUDIO_CLOCK_N)<300) || ((AUDIO_128FS_HZ/AUDIO_CLOCK_N)>1500) )
	begin
		$display("AUDIO_CLOCK_N error");
	end
	//$display("regenerate clock %d Hz (%d ppm)",audioclock_128fs_regen,
	//abs(audioclock_128fs_param/audioclock_128fs_regen)*1000000000000.0);

// video information
$display("VIDEO CODE  = %x",AVIIF_VIC);
$display("VIDEO CLOCK = %f MHz",TMDS_CLOCK_HZ/1000000.0);
// audio information
$display("AUDIO SAMPLING RATE        (rate : 128fs) = %fKHz : %f MHz",AUDIO_128FS_HZ/128000.0,AUDIO_128FS_HZ/1000000.0);
$display("AUDIO CLOCK REGENERATION  (rate : N / CTS) = %fKHz : %d /%d",audioclock_128fs_regen/128000.0,audioclock_n,audioclock_cts);
$display("AUDIO REGENERATION PACKET (rate : inteval) = %fKHz : %d",audio_regen_frequency/1000.0,audio_regen_interval);
$display("AUDIO CAPTURE RATE    (rate : ADD / LIMIT) = %fKHz : %d / %d",(TMDS_CLOCK_HZ/1000.0*AUDIO_TIMER_ADDITION/AUDIO_TIMER_LIMIT),AUDIO_TIMER_ADDITION,AUDIO_TIMER_LIMIT);

$display("AUDIO STREAM CH STATUS = %x",channelStatus);
end

////////////////////////////////////////////////////////////////////////
// local assign
////////////////////////////////////////////////////////////////////////
wire pixclk = clkin;
	
////////////////////////////////////////////////////////////////////////
// stage 1
// Capture Input
////////////////////////////////////////////////////////////////////////

// stage 1 capture data
reg [7:0] rRed1,rBlue1,rGreen1;
reg hSync1, vSync1, DrawArea1;

reg hsync_polarity,vsync_polarity;

always @(posedge pixclk)
begin
	// capture input data
	rRed1     <= red_din;
	rBlue1    <= blue_din;
	rGreen1   <= green_din;
	hSync1    <= hsync;
	vSync1    <= vsync;
	DrawArea1 <= de;
end

////////////////////////////////////////////////////////////////////////
// stage 2
// Analyze / Regenerate Timing Signal
////////////////////////////////////////////////////////////////////////

//
reg [7:0] rRed2,rBlue2,rGreen2;
reg hSync2, vSync2, DrawArea2;
//
reg [11:0] CounterX, CounterY;

// captured timming
reg [11:0] htotal_pos;
reg htotal_matched;
// vertical display end position
reg [11:0] vdispend_pos;
reg  vblank_matched;
reg vblank_captured;  // from DE
reg vblank_generated; // from input & auto

// catpure events
wire hblank_event = {DrawArea2,DrawArea1}==2'b10;
wire htotal_event = {DrawArea2,DrawArea1}==2'b01;
//wire hsync_event  = {hSync2,hSync1}==2'b01; // start HSYNC
wire hsync_event  = (hSync2==hsync_polarity) && (hSync1!=hsync_polarity);

// H-TOTAL
wire htotal_match = (CounterX==htotal_pos);
wire htotal_auto  = htotal_matched && htotal_match;

// VBLANK / VTOTAL
//wire vblank_event = ~vblank && ~htotal_event;
wire vblank_event = ~vblank_captured && htotal_auto && ~htotal_event;
wire vblank_auto  = (vdispend_pos==CounterY); // last 

// VTOTAL return
reg [11:0] vtotal_pos;
wire vtotal_event =  htotal_auto &&  htotal_event && vblank_captured;
wire vtotal_auto  = (CounterY==vtotal_pos);

initial
begin
	CounterX=0;
	CounterY=0;
	hsync_polarity=0;
	vsync_polarity=0;
//
	htotal_pos=0;
	htotal_matched=0;

	vblank_captured=0;
	vblank_generated=0;

	vdispend_pos = 0;
	vtotal_pos = 0;

end

always @(posedge pixclk)
begin
	// to eddge detect
	{rRed2,rBlue2,rGreen2} <= {rRed1,rBlue1,rGreen1};
	{hSync2, vSync2, DrawArea2} <= {hSync1, vSync1, DrawArea1};
	
	///////////////////////////////
	// H TOTAL COUNTER
	CounterX <= CounterX + 1;

	// XCounter return
	if(htotal_event || htotal_auto)
	begin
		CounterX <= 0;
		vblank_captured <= ~htotal_event;
		// capture vblank position
		if(vblank_event)
		begin
			vdispend_pos <= CounterY;
		end
	end
	// Update Auto Return Position
	if(htotal_event)
	begin
		htotal_pos     <= CounterX;
		htotal_matched <= htotal_match;
	end

	///////////////////////////////
	// VBLANK GENERATE
	//
	if(hsync_event)
	begin
		vblank_generated <= (vblank_auto | vblank_captured) & ~vtotal_auto; 
	end
	
	///////////////////////////////
	// V TOTAL COUNTER
	if(vtotal_event)
	begin
		vtotal_pos <= CounterY;
		CounterY <= 0;
	end else if(htotal_event || htotal_auto)
	begin
		CounterY <= CounterY + 1;
	end

	///////////////////////////////
	// capture sync polarity
	if(~DrawArea2 && DrawArea1)
	//if(DrawArea)
	begin
		// CounterX <= 1; // sync as start display
		hsync_polarity <= hSync1; 
		vsync_polarity <= vSync1;
		// syncPolarityChanged <= (hsync_polarity != hSync1) ||  (vsync_polarity != vSync1);

	end

end

/////////////////////////////////////////////////////////
// check sum for infoframe packet
/////////////////////////////////////////////////////////
function [7:0] packetsum;
input [23:0] header;
input [103:0] data;
begin
`ifdef CAUSE_CHECKSUM_ERROR
	packetsum = 8'h55
`else
	packetsum = 8'h00
`endif
	-header[23:16]-header[15:8]-header[7:0]
	-data[103:96]
	-data[95:88]-data[87:80]-data[79:72]-data[71:64]
	-data[63:56]-data[55:48]-data[47:40]-data[39:32]
	-data[31:24]-data[23:16]-data[15: 8]-data[ 7: 0];
end
endfunction

/////////////////////////////////////////////////////////
// 8.2.1 Auxiliary Video information(AVI) InfoFrame
/////////////////////////////////////////////////////////
// Byte0: Checksum
// Byte1: 10 = 0(Y1:Y0=0 RGB)(A0=1 No active format)(B1:B0=00 No bar info)(S1:S0=00 No scan info)
// Byte2: 2A = (C1:C0=0 No colorimetry)(M1:M0=2 16:9)(R3:R0=A 16:9)
// Byte3: 00 = 0(SC1:SC0=0 No scaling)
// Byte4: 03 = 0(VIC6:VIC0=3 720x480p)
// Byte5: 00 = 0(PR5:PR0=0 No repeation)
//					packetHeader<=24'h0D0282;	// infoframe AVI packet
//					subpacket[0]<=56'h000003002A1032;
//					subpacket[1]<=56'h00000000000000;

wire [23:0] aviinfo_header =
{
8'h0D,	// HB2 Length
8'h02,	// HB1 Version
8'h82	// HB0 Packet Type
};
//|PB14-PB27| n.a (0) |
wire [103:0] aviinfo_data =
{
//|PB13| Pixel Number of Start of Right Bar (upper 8 bits)     |
//|PB12| Pixel Number of Start of Right Bar (lower 8 bits)     |
//|PB11| Pixel Number of End of Left Bar (upper 8 bits)        |
//|PB10| Pixel Number of End of Left Bar (lower 8 bits)        |
//|PB9 | Line Number of Start of Bottom Bar (upper 8 bits)     |
//|PB8 | Line Number of Start of Bottom Bar (lower 8 bits)     |
//|PB7 | Line Number of End od Top Bar(upper 8 bits)           |
//|PB6 | Line Number of End od Top Bar(lower 8 bits)           |
	64'h0000_0000_0000_0000,
//|PB5 |F57=0 |F56=0 |F55=0 |F54=0 | PR3  | PR2  | PR1  | PR0  |(Table 12)
	4'b0000,
	4'b0000,	// Pixel Repetition: No Repetition
//|PB4 |F47=0 |VIC6  |VIC5  |VIC4  |VIC3  |VIC2  |VIC1  |VIC0  | BG=03(Table 13)
	1'b0,AVIIF_VIC,
//|PB3 | ITC  |EC2   |EC1   |EC0   | Q1   | Q0   |SC1   |SC0   | BG=00
	1'b0,		// IT content: No data
	3'b000,		// Extended Colorimetry: don't care(xvYCC601)
	2'b00,		// RGB Quantization Range: Default
	2'b00,		// Non-Uniform Picture Scaling:No Known
//|PB2 | C1   | C0   | M1   | M0   | R3   | R2   | R1   | R0   | NG=2A
//					subpacket[0]<=56'h0000_03_00_2A_10;
// C:Colorimetry 
	2'b00,		// NoData
// M:Picture Aspect Ratio
//	2'b10,		// 16:9 NEOGEO
	2'b01,		//  4:3
// R:Active Format Aspect Ratio
//	4'b1010,	// 16:9 CENTER, NEOGEO
//	4'b1001,	//  4:3 CENTER
	4'b1000,	// Same as picture aspect ratio
//|PB1 |F17=0 | Y1   | Y0   | A0   | B1   | B0   | S1   | S0   | NG=10
	1'b0,	// F17
// Y:RGB or YCbCr
	2'b00,	// RGB ,NEOGEO
//	2'b01,	// YCbCr422 for AVI packet TEST
//	2'b10,	// YCbCr444
// A:Acrive Format Information Present
//	1'b1,	// R3..R0 valid ,NEOGEO
	1'b0,	// R3..R0
// B:Bar Info
	2'b00,	// No data ,NEOGEO
// S:Scan Information
	2'b00	// No data ,NEOGEO
};
//|PB0 | Check SUM for packet |
wire [7:0] aviinfo_sum = packetsum(aviinfo_header,aviinfo_data);

/////////////////////////////////////////////////////////
// 8.2.2 Audio InfoFrame
/////////////////////////////////////////////////////////

// Byte0: Checksum
// Byte1: 11 = (CT3:0=1 PCM)0(CC2:0=1 2ch)
// Byte2: 00 = 000(SF2:0=0 As stream)(SS1:0=0 As stream)
// Byte3: 00 = LPCM doesn't use this
// Byte4-5: 00 Multichannel only (>2ch)
//					packetHeader<=24'h0A0184;	// infoframe audio packet
//					subpacket[0]<=56'h00000000001160;
//					subpacket[1]<=56'h00000000000000;

wire [23:0]  audioinfo_header =
{
8'h0A,	// HB2 Length
8'h01,	// HB1 Version
8'h84	// HB0 Packet Type
};
//|PB27| n.a.(0)                                               |
//  :
//|PB14| n.a.(0)                                               |
wire [103:0] audioinfo_data = {
//|PB13| n.a.(0)                                               |
//|PB12| n.a.(0)                                               |
//|PB11| n.a.(0)                                               |
	24'h000000,
//|PB10| Reserved(0)                                           |
//|PB9 | Reserved(0)                                           |
//|PB8 | Reserved(0)                                           |
//|PB7 | Reserved(0)                                           |
//|PB6 | Reserved(0)                                           |
	40'h00_00_00_00_00,
//|PB5 |DM_INH| LSV3 | LSV2 | LSV1 | LSV0 |Rsv(0)|Rsv(0)|Rsv(0)|
	1'b0,		// Downmix inhibit.:Permitted (CEA-861-D table 22)
	4'b0000,	// Level Shift Values: 0db(CEA-861-D table 21)
	3'b000,		// Rsv(0)
//|PB4 | CA7  | CA6  | CA5  | CA4  | CA3  | CA2  | CA1  | CA0  |
	8'h00,		// Channel/Speaker Allocation.: FR,FL (CEA-861-D Section 6.6.2)
//|PB3 | Format depends on coding type (CT0..CT3)              |
	8'h00,
//|PB2 |Rsv(0)|Rsv(0)|Rsv(0)| SF2  | SF1  | SF0  | SS1  | SS0  |(CEA-861-D table 18)
	3'b000,		// Rsv(0)
// SF:Sample Frequency
	3'b000,		// Refer to Stream Header
//	3'b001,		// Sample Frequency: 32KHz
//	3'b011,		// Sample Frequency: 48KHz
// SS:Sample Size.
	2'b00,		// Refer to Stream header
//	2'b01,		// 16bit
//	2'b10,		// 20bit
//	2'b11,		// 24bit
//|PB1 | CT3  | CT2  | CT1  | CT0  | Rsvd | CC2  | CC1  | CC0  |(CEA-861-D table 17)
//CT:Coding Type.
//	4'b0001,	// IEC 60958 PCM[30,31] , NEOGEO
	4'b0000,	// Refer to Stream Header
	1'b0,		// Rsv(0)
// CC:Chanel Count.
//	3'b001		// 2ch , NEOGEO
	3'b000		// Refer to Stream Header
};
//|PB0 | Check SUM for packet |
wire [7:0]   audioinfo_sum = packetsum(audioinfo_header,audioinfo_data);

/////////////////////////////////////////////////////////
// 5.3.3 Audio Clock Regeneration Packet
/////////////////////////////////////////////////////////

wire [23:0]  audioclock_header =
{
8'h00,8'h00,8'h01	// HB2,HB1,HB0
};
wire [55:0] audioclock_subpacket = {
// |SB6| N.7  | N.6  | N.5  | N.4  | N.3  | N.2  | N.1  | N.0  |
audioclock_n[7:0],
// |SB5| N.15 | N.14 | N.13 | N.12 | N.11 | N.10 | N.9  | N.8  |
audioclock_n[15:8],
// |SB4|  0   |  0   |  0   |  0   | N.19 | N.18 | N.17 | N.16 |
4'b0000,audioclock_n[19:16],
// |SB3|CTS.75|CTS.6 |CTS.5 |CTS.4 |CTS.3 |CTS.2 |CTS.1 |CTS.0 |
audioclock_cts[7:0],
// |SB2|CTS.15|CTS.14|CTS.13|CTS.12|CTS.11|CTS.10|CTS.9 |CTS.8 |
audioclock_cts[15:8],
// |SB1|  0   |  0   |  0   |  0   |CTS.19|CTS.18|CTS.17|CTS.16|
4'b0000,audioclock_cts[19:16],
// |SB0|  0   |  0   |  0   |  0   |  0   |  0   |  0   |  0   |
8'h00
};

////////////////////////////////////////////////////////////////////////
// HDMI audio packet generator
//
// 5.3.4 Audio Sample Packet
//
////////////////////////////////////////////////////////////////////////
reg [23:0] curSampleL;
reg [23:0] curSampleR;

reg [23:0] audioPacketHeader;
reg [55:0] audioSubPacket[3:0];
reg [7:0] channelStatusIdx;

reg [19:0] audioTimer;
reg [ 6:0] audioCount;

reg [9:0] audioSamples;
reg sendRegenPacket;
//reg [1:0] samplesHead;
reg audioCaptureRequest;


// Table 5-12 Audio Sample Packet Header
// |HB2| B.3  | B.0  | B.1  | B.0  |flat.3|flat.2|flat.1|flat.0|
// |HB1|  0   |  0   |  0   |layout| sp.3 | sp.2 | sp.1 | sp.0 |
// |HB0|  0   |  0   |  0   |  0   |  0   |  0   |  1   |  0   |
`define OFS_SP0 (1*8+0)
`define OFS_SP1 (1*8+1)
`define OFS_SP2 (1*8+2)
`define OFS_SP3 (1*8+3)
`define OFS_BLK0 (2*8+4)
`define OFS_BLK1 (2*8+5)
`define OFS_BLK2 (2*8+6)
`define OFS_BLK3 (2*8+7)

wire [1:0] samples_held = 
	~audioPacketHeader[`OFS_SP0] ? 0 :
	~audioPacketHeader[`OFS_SP1] ? 1 :
	~audioPacketHeader[`OFS_SP2] ? 2 : 3;

// Table 5-13 Audio Sample Subpacket
wire chs_block = (channelStatusIdx==0);
wire chs_dataL = channelStatus[channelStatusIdx] | (channelStatusIdx==20); // Ch.0
wire chs_dataR = channelStatus[channelStatusIdx] | (channelStatusIdx==21); // Ch.1
wire udata_bit = 1'b0;
wire valid_bit = 1'b1;
wire parity_l  = (^curSampleL)^chs_dataL^udata_bit^valid_bit;
wire parity_r  = (^curSampleR)^chs_dataR^udata_bit^valid_bit;

wire [55:0] audioSamplingSubPacketData = {
// |SB6|  PR  |  CR  |  UR  |  VR  |  PL  |  CL  |  UL  |  VL  |
	parity_r,chs_dataR,udata_bit,valid_bit,
	parity_l,chs_dataL,udata_bit,valid_bit,
// |SB5| R.27 |  -   |  -   |  -   |  -   |  -   |  -   | R.20 |
// |SB4| R.19 |  -   |  -   |  -   |  -   |  -   |  -   | R.12 |
// |SB3| R.11 |  -   |  -   |  -   |  -   |  -   |  -   | R.4  |
	curSampleR[23:0],
// |SB2| L.27 |  -   |  -   |  -   |  -   |  -   |  -   | L.20 |
// |SB1| L.19 |  -   |  -   |  -   |  -   |  -   |  -   | L.12 |
// |SB0| L.11 |  -   |  -   |  -   |  -   |  -   |  -   | L.4  |
	curSampleL[23:0]
};

// hold packet data in data island phase
wire audio_transmit_phase;
reg audio_transmit_phase_d;

initial
begin
	curSampleL=0;
	curSampleR=0;

	audioPacketHeader=0;
	audioSubPacket[0]=0;
	audioSubPacket[1]=0;
	audioSubPacket[2]=0;
	audioSubPacket[3]=0;
	channelStatusIdx=0;
	audioTimer=0;
	audioCount=0;
	audioSamples=0;
//	samplesHead=0;
	sendRegenPacket=0;
//
	audioCaptureRequest=0;
	audio_transmit_phase_d=0;
end

//wire audioSamplingTriger = (!samplesHeadClear) & (audioTimer>=AUDIO_TIMER_LIMIT);

// audio clock generation
wire ev_fs128 = (audioTimer >= audioclock_cts);
wire ev_fs1   = ev_fs128 & (audioCount==7'h7f);
always @(posedge pixclk)
begin
	if(ev_fs128)
	begin
		// 128fs event
		audioTimer <= audioTimer + audioclock_n - audioclock_cts;
		audioCount <= audioCount + 7'd1;
	end else begin
		audioTimer <= audioTimer + audioclock_n;
	end
end
// output signal
assign auido_1fs   = ev_fs1;
assign auido_128fs = ev_fs128;

//task AudioPacketGeneration;
always @(posedge pixclk)
begin
	if(ev_fs1)
	begin
		// 1fs , capture audio input
		curSampleL <= audio_l;
		curSampleR <= audio_r;
		// request to store data packet
		audioCaptureRequest <= 1'b1;
	end

	// capture / transmit phase control
	audio_transmit_phase_d <= audio_transmit_phase;
	//
	if(audioCaptureRequest && ~audio_transmit_phase_d)
	begin
		// recept audio capture cycle
		audioCaptureRequest <= 1'b0;

		// generate audio clock regeneration packet request
		// 1KHz beacon packet (32 times -> 1KHz)
		audioSamples<=audioSamples+1;
		if(audioSamples[6:0]==(audio_regen_interval-1) )
		begin
			audioSamples <= 0;
			sendRegenPacket<=1;
		end
		
		// store fixed header
		audioPacketHeader[7:0] <= 8'h02;
		// store subpacket & mark to header
		audioSubPacket[samples_held]<=audioSamplingSubPacketData;
		// mark subpacket to header
		if(~audioPacketHeader[`OFS_SP0]) begin
			audioPacketHeader[`OFS_SP0]  <= 1'b1;      // valid flag
			audioPacketHeader[`OFS_BLK0] <= chs_block; // block flag
		end else if(~audioPacketHeader[`OFS_SP1]) begin
			audioPacketHeader[`OFS_SP1] <= 1'b1;
			audioPacketHeader[`OFS_BLK1] <= chs_block;
		end else if(~audioPacketHeader[`OFS_SP2]) begin
			audioPacketHeader[`OFS_SP2] <= 1'b1;
			audioPacketHeader[`OFS_BLK2] <= chs_block;
		end else begin
			audioPacketHeader[`OFS_SP3] <= 1'b1;
			audioPacketHeader[`OFS_BLK3] <= chs_block;
		end
		// update channel status
		channelStatusIdx <= (channelStatusIdx<191) ? (channelStatusIdx+1) : 0;
	end

	// clear packet data
	if(audio_transmit_phase_d & ~audio_transmit_phase)
	begin
		// end of transmit phase
		sendRegenPacket<=0;
		audioPacketHeader <= 24'h000000;
		audioSubPacket[0] <= 56'h00000000000000;
		audioSubPacket[1] <= 56'h00000000000000;
		audioSubPacket[2] <= 56'h00000000000000;
		audioSubPacket[3] <= 56'h00000000000000;
	end
end
//endtask

////////////////////////////////////////////////////////////////////////
// 5.2.3.5 Data Island Error Correction Code
//
// Error correction code generator
// Generates error correction codes needed for verifying HDMI packets
////////////////////////////////////////////////////////////////////////

// Figure 5-5 Error Correction Code generator
function [7:0] ECCcode;	// Cycles the error code generator
	input [7:0] code;
	input bita;
	input passthroughData;
	begin
`ifdef CAUSE_ECC_ERROR
		ECCcode = {code[6:0],code[7]^1}; // ERROR
`else
		ECCcode = (code<<1) ^ (((code[7]^bita) && passthroughData)?(1+(1<<6)+(1<<7)):0);
`endif
	end
endfunction

/*
2stage ECC calcration
*/
function [7:0] ECC2u;
input [7:0] code;
input [1:0] data;
input passthroughData;
begin
	ECC2u = ECCcode(ECCcode(code, data[0], passthroughData), data[1], passthroughData);
end
endfunction

////////////////////////////////////////////////////////////////////////
// Packet sending
// During hsync periods send audio data and infoframe data packets
////////////////////////////////////////////////////////////////////////

// 4b10b data
reg [3:0] dataChannel0;
reg [3:0] dataChannel1;
reg [3:0] dataChannel2;
//
wire [23:0] packetHeader;
wire [55:0] subpacket[3:0];
//reg [23:0] packetHeader;
//reg [55:0] subpacket[3:0];
reg [7:0] bchHdr;
reg [7:0] bchCode [3:0];
reg [4:0] dataOffset;
// 2b10b data
reg [3:0] preamble;
reg [1:0] ctrl3;
// 2b10b data
reg tercData;
reg dataGuardBand;
reg videoGuardBand;
reg ctrlSel3;
initial
begin
	dataChannel0=0;
	dataChannel1=0;
	dataChannel2=0;
	//packetHeader=0;
	//subpacket[0]=0;
	//subpacket[1]=0;
	//subpacket[2]=0;
	//subpacket[3]=0;
	bchHdr=0;
	bchCode[0]=0;
	bchCode[1]=0;
	bchCode[2]=0;
	bchCode[3]=0;
	dataOffset=0;
	preamble=0;
	ctrl3=0;
	tercData=0;
	dataGuardBand=0;
	videoGuardBand=0;
end

// task SendPacket;
/////////////////////////////////////////////////////////
// data packet selection / arbiter
/////////////////////////////////////////////////////////

// data packet selector
reg [1:0] packet_sel;
localparam PACKET_AVIINFO   = 0;
localparam PACKET_AUDIOINFO = 1;
localparam PACKET_CLOCKREGENERATION = 2;
localparam PACKET_AUDIO_SAMPLE = 3;

// data packet selector
wire [247:0] dataPacketMux = 
	packet_sel==0 ? {aviinfo_header,56'h00000000000000,56'h00000000000000,aviinfo_data,aviinfo_sum} :
	packet_sel==1 ? {audioinfo_header,56'h00000000000000,56'h00000000000000,audioinfo_data,audioinfo_sum} :
	packet_sel==2 ? {audioclock_header,{4{audioclock_subpacket}}} :
	{audioPacketHeader,audioSubPacket[3],audioSubPacket[2],audioSubPacket[1],audioSubPacket[0]};
assign packetHeader = dataPacketMux[247:224]; 
assign subpacket[3] = dataPacketMux[223:168]; 
assign subpacket[2] = dataPacketMux[167:112];
assign subpacket[1] = dataPacketMux[111:56]; 
assign subpacket[0] = dataPacketMux[55:0];

/////////////////////////////////////////////////////////
// Signal Generation
/////////////////////////////////////////////////////////

// 2bit data from subpacket/ecc
function [1:0] subpacketSelect;
input [55:0] subpacket;
input [7:0] ecc;
input [4:0] index;
begin
	subpacketSelect = index<5'd28 ? {subpacket[index*2+1],subpacket[index*2]} : {ecc[6],ecc[7]};
end
endfunction

// 2bits pickup from subpacket
wire [1:0] subp0 = subpacketSelect(subpacket[0],bchCode[0],dataOffset);
wire [1:0] subp1 = subpacketSelect(subpacket[1],bchCode[1],dataOffset);
wire [1:0] subp2 = subpacketSelect(subpacket[2],bchCode[2],dataOffset);
wire [1:0] subp3 = subpacketSelect(subpacket[3],bchCode[3],dataOffset);
// Data / ECC select
wire packetHeaderPhase = dataOffset<24; // 8T ECC
wire packetDataPhase   = dataOffset<28; // 4T ECC

/////////////////////////////////////////////////////////
// output data phase control
/////////////////////////////////////////////////////////

// cycles
//`define PREAMBLE_SPACE	4
`define PREAMBLE_SPACE	8
`define DATA_PREAMBLE	8
`define DATA_GUARDBAND	2
`define DATA_SIZE		32
// Pixel Data Island
`define VIDEO_PREAMBLE	8
`define VIDEO_GUARDBAND	2

// state machine
reg [3:0] phase;
localparam PHASE_VIDEO_DATA       = 0;
localparam PHASE_DATA_SPACING     = 1;
localparam PHASE_DATA_PREAMBLE    = 2;
localparam PHASE_DATA_GUARDBAND1  = 3;
localparam PHASE_DATA_DATAPACKET1 = 4;
localparam PHASE_DATA_DATAPACKET2 = 5;
localparam PHASE_DATA_GUARDBAND2  = 6;
localparam PHASE_VIDEO_SPACING    = 7;
localparam PHASE_VIDEO_PREAMBLE   = 8;
localparam PHASE_VIDEO_GUARDBAND  = 9;
localparam PHASE_DONE             = 10;

//
assign audio_transmit_phase = (phase >= PHASE_DATA_GUARDBAND1) && (phase <=PHASE_DATA_GUARDBAND2);

// 
function [4:0] phaseEndCount;
input [3:0] phase;
begin
	case(phase)
	PHASE_VIDEO_DATA       : phaseEndCount = 0;
	PHASE_DATA_SPACING     : phaseEndCount = `PREAMBLE_SPACE-1;
	PHASE_DATA_PREAMBLE    : phaseEndCount = `DATA_PREAMBLE-1;
	PHASE_DATA_GUARDBAND1  : phaseEndCount = `DATA_GUARDBAND-1;
	PHASE_DATA_DATAPACKET1 : phaseEndCount = `DATA_SIZE-1;
	PHASE_DATA_DATAPACKET2 : phaseEndCount = `DATA_SIZE-1;
	PHASE_DATA_GUARDBAND2  : phaseEndCount = `DATA_GUARDBAND-1;
	PHASE_VIDEO_SPACING    : phaseEndCount = `PREAMBLE_SPACE-1;
	PHASE_VIDEO_PREAMBLE   : phaseEndCount = `VIDEO_PREAMBLE-1;
	PHASE_VIDEO_GUARDBAND  : phaseEndCount = `VIDEO_GUARDBAND-1;
	PHASE_DONE             : phaseEndCount = 0;
	default:
		phaseEndCount = 0;
	endcase
end
endfunction

// last cycle in phase
wire phase_next = dataOffset>=phaseEndCount(phase);

// total count
wire [11:0] dataPhaseTotalCount=
	`VIDEO_GUARDBAND+`VIDEO_PREAMBLE+`PREAMBLE_SPACE+
	`DATA_GUARDBAND+`DATA_SIZE+`DATA_SIZE+`DATA_GUARDBAND+
	`DATA_PREAMBLE+`PREAMBLE_SPACE;

// phase control
always @(posedge pixclk)
begin
	// through 2b/8b selector
	ctrl3    <= {vSync2, hSync2};
	ctrlSel3 <= DrawArea2;
	//videoGuardBand <= (phase==PHASE_VIDEO_GUARDBAND);
	//dataGuardBand <= (phase==PHASE_DATA_GUARDBAND1) || (phase==PHASE_DATA_GUARDBAND2);
	//dataGuardBand <= (phase==PHASE_DATA_GUARDBAND1) || (phase==PHASE_DATA_GUARDBAND2);
	//tercData <= (phase>=PHASE_DATA_GUARDBAND1) && (phase<=PHASE_DATA_GUARDBAND2);

	/// phase control
	if(CounterX<(htotal_pos-dataPhaseTotalCount))
	begin
		// restart phase and hold until start position
		phase <= PHASE_VIDEO_DATA;
		dataOffset<=0;
	end else if(phase==PHASE_DONE)
	begin
		// hold end of region
	end else if(phase_next)
	begin
		// next phase
		phase <= phase + 1;
		dataOffset<=0;
	end else begin
		// continue phase and countup
		dataOffset <= dataOffset +1;
	end

`ifdef DISABLE_DATA_ISLAND
	phase <= PHASE_VIDEO_DATA;
`endif // DISABLE_DATA_ISLAND

	// TMDS region control
	case(phase)
	PHASE_DATA_SPACING: // SPACING of VIDEO DATA to DATA PREAMBLE
	begin
		// check blank area
		// if(DrawArea2) datastart_nonblanking_error <= 1;
	end
	PHASE_DATA_PREAMBLE:
	begin
		// Send the data period preamble
		// A nice "feature" of my test monitor (GL2450) is if you comment out
		// this line you see your data next to your image which is useful for
		// debugging
		preamble<='b0101;
	end
	PHASE_DATA_GUARDBAND1:
	begin
		// Start sending leading data guard band
		tercData<=1;
		dataGuardBand<=1;
		dataChannel0<={1'b1, 1'b1, vSync1, hSync1};
		preamble<=0;
		// Set up the first of the packets we'll send
		if(phase_next)
		begin
			if (sendRegenPacket) begin
				// audio clock regeneration packet
				packet_sel <= PACKET_CLOCKREGENERATION;
				// sendRegenPacket<=0;
			end else begin
				if (!CounterY[0]) begin
					packet_sel <= PACKET_AVIINFO;
				end else begin
					packet_sel <= PACKET_AUDIOINFO;
				end
			end
		end
	end
	PHASE_DATA_DATAPACKET1,PHASE_DATA_DATAPACKET2:
	begin
		dataGuardBand<=0;
		// SendPacket : Send data packet (Infoframe or audio clock regen)
		//   1st : AVI Infoframe / AUDIO Infoframe / audio clock regen
		//   2nd : Audio Sample Data / NUL
		dataChannel0[0]<=hSync1;
		dataChannel0[1]<=vSync1;
		dataChannel0[3]<=dataGuardBand?1'b0:1'b1; // only 1st bit in 1st packet = 0
		// output header
		dataChannel0[2]<=packetHeaderPhase?packetHeader[dataOffset]:bchHdr[7]; // header or ECC
		// output data/ecc
		{dataChannel2[0], dataChannel1[0]} <= subp0; // data or ECC
		{dataChannel2[1], dataChannel1[1]} <= subp1; // data or ECC
		{dataChannel2[2], dataChannel1[2]} <= subp2; // data or ECC
		{dataChannel2[3], dataChannel1[3]} <= subp3; // data or ECC

		// ECC code generation
		bchHdr     <= ECCcode(bchHdr, packetHeader[dataOffset], packetHeaderPhase);
		bchCode[0] <= ECC2u(bchCode[0], subp0, packetDataPhase);
		bchCode[1] <= ECC2u(bchCode[1], subp1, packetDataPhase);
		bchCode[2] <= ECC2u(bchCode[2], subp2, packetDataPhase);
		bchCode[3] <= ECC2u(bchCode[3], subp3, packetDataPhase);

		// update Next Data
		if(phase_next) 
		begin
			// load 2nd data packet (audio sampling data , or NUL)
			packet_sel <= PACKET_AUDIO_SAMPLE;
			// end of audio sample packet
		end
//		dataOffset<=dataOffset+5'b1;
	end
	PHASE_DATA_GUARDBAND2:
	begin
		// Trailing guardband for data period
		dataGuardBand<=1;
		dataChannel0<={1'b1, 1'b1, vSync1, hSync1};	
	end
	PHASE_VIDEO_SPACING:
	begin
		// end of data guardband
		tercData<=0;
		dataGuardBand<=0;
		preamble<='b0000;
	end
	PHASE_VIDEO_PREAMBLE:
	begin
		if(~vblank_generated) // non-generate in vblank
			preamble<='b0001; // Video Priamble
	end
	PHASE_VIDEO_GUARDBAND:
	begin
		// end of preamble
		preamble<=0;
		if(~vblank_generated) // non-generate in vblank
			videoGuardBand<=1;
	end
	PHASE_DONE:
	begin
		// end of guardband
		videoGuardBand<=0;
		// check draw area
		// if(~DrawArea2) videodata_sync_error <= 1;
	end
	endcase
end

////////////////////////////////////////////////////////////////////////
// 10b encoder
// Encodes video data (TMDS) or packet data (TERC4) ready for sending
////////////////////////////////////////////////////////////////////////

// HDMI-CH2 / R
encodeh #(
	.vgbpattern(10'b1011001100),
	.dgbpattern(10'b0100110011)
) ch0enc10b  (
	.rstin(rstin),.clkin(pixclk),
	.din(rRed2),.cin(preamble[3:2]),.de(ctrlSel3),
	.terc4in(dataChannel2),.t4en(tercData),.vgben(videoGuardBand),.dgben(dataGuardBand),
	.dout(tmds_data2)
);
// HDMI-CH1 / G
encodeh #(
	.vgbpattern(10'b0100110011),
	.dgbpattern(10'b0100110011)
) ch1enc10b  (
	.rstin(rstin),.clkin(pixclk),
	.din(rGreen2),.cin(preamble[1:0]),.de(ctrlSel3),
	.terc4in(dataChannel1),.t4en(tercData),.vgben(videoGuardBand),.dgben(dataGuardBand),
	.dout(tmds_data1)
);
// HDMI-CH0 / B
encodeh #(
	.vgbpattern(10'b1011001100),
	.dgbpattern(10'b0100110011)
) ch2enc10b  (
	.rstin(rstin),.clkin(pixclk),
	.din(rBlue2),.cin(ctrl3),.de(ctrlSel3),
	.terc4in(dataChannel0),.t4en(tercData),.vgben(videoGuardBand),.dgben(1'b0),
	.dout(tmds_data0)
);
endmodule

////////////////////////////////////////////////////////////////////////
