/****************************************************************************
viode capture parameter selector

2021.3 Tatsuyuki Sato

note:

****************************************************************************/
module capture_parameter
#(
  parameter PCLK_DIV  = 27000000,
  parameter DIV_DEPTH = 26
)
(
//  input I_RESET,
  input I_CLK,
  input [2:0]  I_SEL,
  input [11:0] I_WIDTH,
  input [11:0] I_HEIGHT,
// clock regenerate parameter
  output O_PCLK_REGEN,					// pclk renegerate enable
  output [DIV_DEPTH-1:0] O_PCLKDIV,		// pclk divisor
  output [DIV_DEPTH-1:0] O_PCLKMUL,		// pclk multiplier
  output [DIV_DEPTH-1:0] O_PCLKPHASE,	// pclk hsync-phase
// data enable regenerate parameter
  output O_DE_REGEN,       	  			// de   renegerate enable
  output [11:0] O_HSTART,				// horizonal start position
  output [11:0] O_HSIZE,				// horizonal window size
  output [11:0] O_VSTART,				// vertical start position
  output [11:0] O_VSIZE,				// vertical window size
// scaler parameter
  output [11:0] O_HSCALE,				//
  output [11:0] O_VSCALE				//
);

// number of total parameters
localparam param_size = (DIV_DEPTH+DIV_DEPTH) +(12+12+12) +(12+12+12);

//******************************************************************//
// scale parameter
//******************************************************************//

function [DIV_DEPTH*2-1:0] CLOCK_PHASE(input integer clock_val,input integer phase_val);
begin
    CLOCK_PHASE[DIV_DEPTH*2-1:DIV_DEPTH]  =  clock_val;
    CLOCK_PHASE[DIV_DEPTH  -1:        0]  =  phase_val;
end
endfunction

function [35:0] START_SIZE_SCALE(input integer start_val,input integer size_val,input real scale_val);
begin
    START_SIZE_SCALE[35:24] = start_val;
    START_SIZE_SCALE[23:12] =  size_val;
    START_SIZE_SCALE[11: 0] = 256.0 * scale_val;
end
endfunction


//******************************************************************//
// RX62N
//******************************************************************//
wire [param_size-1:0] rx62_parameter = 
{
//  CLOCK_PHASE(26'd800000 ,  26'0)  , // PCLK regenerate
  CLOCK_PHASE(0,0),                  // PCLK ext-input
  START_SIZE_SCALE(0 , 0 , 2.0),     // H:
  START_SIZE_SCALE(0 , 0 , 2.0)      // V:
};

//******************************************************************//
// SH2A
//******************************************************************//
wire [param_size-1:0] sh2a_parameter = rx62_parameter;

//******************************************************************//
// BLANCA
//******************************************************************//
wire [param_size-1:0] blnc_parameter = 
{
  CLOCK_PHASE(0,0),                  // PCLK ext-input
  START_SIZE_SCALE(0 , 0 , 1.0),    // H:
  START_SIZE_SCALE(0 , 0 , 1.0)     // V:
};

/****************************************************************************
// X1 24KHz 3bpp RGB
****************************************************************************/
wire [DIV_DEPTH-1:0]  x1hi_phase =  (PCLK_DIV*116/100); //(PCLK_DIV*114/100) ; // sampling skew , -0.20ui

wire [param_size-1:0] x1hi_parameter = 
{
  CLOCK_PHASE(26'd2147727   ,  x1hi_phase), // PCLK regen   : clock M , clock phase
  START_SIZE_SCALE(98 , 640 , 1.0),    // H:
  START_SIZE_SCALE(22 , 400 , 1.0)     // V:
};

//******************************************************************//
// X1 15KHz 3bpp RGB X1
//******************************************************************//
//wire [DIV_DEPTH-1:0] x1lo_phase =  PCLK_DIV*90/100; // skew , sampline = HSYNC.RAISE -0.2UI

//wire [DIV_DEPTH-1:0] x1lo_phase =  PCLK_DIV*50/100; // tuebo OK
wire [DIV_DEPTH-1:0] x1lo_phase =  PCLK_DIV*25/100; // X1 test
//wire [DIV_DEPTH-1:0] x1lo_phase =  PCLK_DIV*75/100; // x1 test
  
// skew , sampline = HSYNC.RAISE -0.2UI
//assign hstart     = (111-91)*8 -28 -1;
//assign vstart     = 31-28)*8-3+2 -20;
wire [param_size-1:0] x1lo_parameter = 
{
  CLOCK_PHASE(26'd1431818, x1lo_phase),           // PCLK regen   : clock M , clock phase
  START_SIZE_SCALE(131-8 , 640+16 , 1.0),    // H:
  START_SIZE_SCALE( 30-8 , 200+16 , 2.0)     // V:
};

//******************************************************************//
// input mode selector
//******************************************************************//

// multiplex
wire [param_size-1:0] mux_parameter;

assign mux_parameter = 
  I_SEL=='d1 ? rx62_parameter :
  I_SEL=='d2 ? sh2a_parameter :
  I_SEL=='d3 ? blnc_parameter :
  I_SEL=='d4 ? x1hi_parameter :
  I_SEL=='d5 ? x1lo_parameter :
  { CLOCK_PHASE(0,0) , START_SIZE_SCALE(0 , 0 , 2.0) , START_SIZE_SCALE(0 , 0 , 2.0) }; // test patern
// hold parameter

// larch parameter
reg pclk_regen;
reg de_regen;
reg  [param_size-1:0] reg_parameter;

always @(posedge I_CLK)
begin
  reg_parameter <= mux_parameter;
  pclk_regen    <= (O_PCLKMUL != 'd0);
  de_regen      <= (O_HSIZE   != 'd0);
end

//******************************************************************//
// output assing
//******************************************************************//
//
assign O_PCLKMUL    = reg_parameter[DIV_DEPTH+DIV_DEPTH+71:DIV_DEPTH+72];
assign O_PCLKPHASE  = reg_parameter[DIV_DEPTH+71:72];
//
assign O_HSTART     = reg_parameter[71:60];
assign O_HSIZE      = reg_parameter[59:48];
assign O_HSCALE     = reg_parameter[47:36];
assign O_VSTART     = reg_parameter[35:24];
assign O_VSIZE      = reg_parameter[23:12];
assign O_VSCALE     = reg_parameter[11: 0];
//
assign O_PCLKDIV    = PCLK_DIV;
assign O_PCLK_REGEN = pclk_regen;
assign O_DE_REGEN   = de_regen;

endmodule
