/****************************************************************************
PMOD x4 LVTTL video capture

2021.3 Tatsuyuki Sato

note:

****************************************************************************/

`timescale 1 ps / 1 ps

`define SEL_INTERNAL    'd0
`define SEL_SH2A        'd1
`define SEL_RX62N       'd2
`define SEL_BLANCA_CN18 'd3

/****************************************
 TOP module
****************************************/
module pmod_capture #
(
  parameter PCLK_DIV  = 27000000,
  parameter DIV_DEPTH = 26
) (
  input  wire [2:0] I_SEL,  // 0=SH2A,1=RX62N,3=BLANCA_CN18
  input  wire       I_CLK,
// internal signal
  input  wire [7:0] I_G,I_R,I_B,
  input  wire       I_DE,
  input  wire       I_VSYNC,
  input  wire       I_HSYNC,
// external PIN
  input  wire [7:0] I_J0,
  input  wire [7:0] I_J1,
  input  wire [7:0] I_J2,
  input  wire [7:0] I_J3,
// clock regenerate parameter
  output O_PCLK_REGEN,					// pclk renegerate enable
  output [DIV_DEPTH-1:0] O_PCLKDIV,		// pclk divisor
  output [DIV_DEPTH-1:0] O_PCLKMUL,		// pclk multiplier
  output [DIV_DEPTH-1:0] O_PCLKPHASE,	// pclk hsync-phase
// data enable regenerate parameter
  output O_DE_REGEN,       	  			// de   renegerate enable
  output [11:0] O_HSTART,				// horizonal start position
  output [11:0] O_HSIZE,				// horizonal window size
  output [11:0] O_VSTART,				// vertical start position
  output [11:0] O_VSIZE,				// vertical window size
// scaler parameter
  output [11:0] O_HSCALE,				//
  output [11:0] O_VSCALE,				//
// video signal
  output wire [7:0] O_G,O_R,O_B,
  output wire       O_CLK,
  output wire       O_DE,
  output wire       O_VSYNC,
  output wire       O_HSYNC
);

/****************************************
input
****************************************/
wire [7:0] j0 = I_J0;
wire [7:0] j1 = I_J1;
wire [7:0] j2 = I_J2;
wire [7:0] j3 = I_J3;

/****************************************
capture parameter
****************************************/
capture_parameter #(
  .PCLK_DIV(PCLK_DIV) , .DIV_DEPTH(DIV_DEPTH)
) capture_parameter_inst (
//  .I_RESET(),
  .I_CLK(I_CLK),
  .I_SEL(I_SEL),
  .I_WIDTH(0),   .I_HEIGHT(0),
// clock regenerate parameter
  .O_PCLK_REGEN(O_PCLK_REGEN),	// pclk renegerate enable
  .O_PCLKDIV(O_PCLKDIV),		// pclk divisor
  .O_PCLKMUL(O_PCLKMUL),		// pclk multiplier
  .O_PCLKPHASE(O_PCLKPHASE),	// pclk hsync-phase
// data enable regenerate parameter
  .O_DE_REGEN(O_DE_REGEN), 	  		// de   renegerate enable
  .O_HSTART(O_HSTART),				// horizonal start position
  .O_HSIZE(O_HSIZE),				// horizonal window size
  .O_VSTART(O_VSTART),				// vertical start position
  .O_VSIZE(O_VSIZE),				// vertical window size
// scaler parameter
  .O_HSCALE(O_HSCALE),
  .O_VSCALE(O_VSCALE)
);

/****************************************
CQ SH2A/RX62N 16bpp to 24bpp conversion
****************************************/
function [23:0] data16grb24(input [15:0] data);
begin
  data16grb24[15: 8] = {data[15:11],{3{data[15]}} }; // R5 to R8
  data16grb24[23:16] = {data[10: 5],{2{data[10]}} }; // G6 to G8 
  data16grb24[ 7: 0] = {data[ 4: 0],{3{data[ 4]}} }; // B5 to B8
end
endfunction

/****************************************
SH2A pin assign (U/D swap from RX62N)
****************************************/
wire [15:0] sh2a_d;
wire        sh2a_clk , sh2a_de , sh2a_vs , sh2a_hs;
wire [ 7:0] sh2a_g , sh2a_r , sh2a_b;

assign sh2a_d = {
  j1[3],j1[7],j1[2],j1[6],j1[1],j1[5],j1[0],j1[4],
  j0[3],j0[7],j0[2],j0[6],j0[1],j0[5],j0[0],j0[4]
};
assign sh2a_de = j2[0];
assign sh2a_vs = j2[1];
assign sh2a_clk= j2[4];
assign sh2a_hs = j2[5];

assign {sh2a_g,sh2a_r,sh2a_b} = data16grb24(sh2a_d); // 16bpp to 24bpp

/****************************************
RX62N pin assign
****************************************/
wire [15:0] rx62_d;
wire        rx62_clk , rx62_de , rx62_vs ,rx62_hs;
wire [ 7:0] rx62_g , rx62_r , rx62_b;

assign rx62_d = {
  j1[7],j1[3],j1[6],j1[2],j1[5],j1[1],j1[4],j1[0],
  j0[7],j0[3],j0[6],j0[2],j0[5],j0[1],j0[4],j0[0]
};
assign rx62_de =  j2[0]; // negative
assign rx62_vs =  j2[1]; // negative
assign rx62_clk=  j2[4];
assign rx62_hs =  j2[5]; // negative

assign {rx62_g,rx62_r,rx62_b} = data16grb24(rx62_d); // 16bpp to 24bpp

/****************************************
BLANCA pin assign
****************************************/
wire        blan_clk;
wire        blan_de;
wire        blan_vs;
wire        blan_hs;
wire [ 7:0] blan_g , blan_r , blan_b;

assign blan_r  = { j0[0],j0[1],j0[2],j0[3],j0[4],j0[5],j0[6],j0[7] };
assign blan_g  = { j1[0],j1[1],j1[2],j1[3],j1[4],j1[5],j1[6],j1[7] };
assign blan_b  = { j2[0],j2[1],j2[2],j2[3],j2[4],j2[5],j2[6],j2[7] };
assign blan_vs = ~j3[2];
assign blan_clk= ~j3[3];
assign blan_de =  j3[6];
assign blan_hs = ~j3[7];

/****************************************
DIGITAL RGB 3bpp
****************************************/
wire        rgb3_clk;
wire        rgb3_de;
wire        rgb3_vs;
wire        rgb3_hs;
wire [ 7:0] rgb3_g , rgb3_r , rgb3_b;

assign rgb3_g  = {8{ j2[2] }};
assign rgb3_r  = {8{ j2[7] }};
assign rgb3_b  = {8{ j2[0] }};
assign rgb3_vs = ~j2[1];
assign rgb3_hs = ~j2[3];
assign rgb3_clk= 1'b0;
assign rgb3_de = 1'b0;

/****************************************
input selector
****************************************/
wire [4+24 -1:0] mux;
assign mux =
  I_SEL=='d1 ? {sh2a_clk , sh2a_de , sh2a_hs , sh2a_vs , sh2a_g,sh2a_r,sh2a_b} :
  I_SEL=='d2 ? {rx62_clk , rx62_de , rx62_hs , rx62_vs , rx62_g,rx62_r,rx62_b} :
  I_SEL=='d3 ?  {blan_clk , blan_de , blan_hs , blan_vs , blan_g,blan_r,blan_b} :
  I_SEL=='d4 ? {rgb3_clk , rgb3_de , rgb3_hs , rgb3_vs , rgb3_g,rgb3_r,rgb3_b} :
  I_SEL=='d5 ? {rgb3_clk , rgb3_de , rgb3_hs , rgb3_vs , rgb3_g,rgb3_r,rgb3_b} :
               {I_CLK    , I_DE   , I_HSYNC  , I_VSYNC , I_G   ,I_R   ,I_B   } ;

/****************************************
output assign
****************************************/
assign O_CLK   = mux[27];
assign O_DE    = mux[26];
assign O_HSYNC = mux[25];
assign O_VSYNC = mux[24];
assign O_G     = mux[23:16];
assign O_R     = mux[15: 8];
assign O_B     = mux[ 7: 0];

endmodule
