//******************************************************************//
// VIDEO CAPTURE
//******************************************************************//

`timescale 1 ps / 1 ps

module videocapture
#(
	parameter DATA_SIZE     = 320*200,	// maximum pixels
	parameter ADDR_DEPTH	= 16,		// address depth
	parameter DATA_DEPTH	= 16		// data depth
)(
	input  wire	I_RESET,
	// PIXEL INPUT DATA
	input  wire	I_CLK,
	input  wire	I_DISP,
	input  wire	I_VSYNC,
	input  wire	[DATA_DEPTH-1:0] I_PIXEL,
	// STATUS
	output reg		[11:0]       O_WIDTH,
	output reg		[11:0]       O_HEIGHT,
	// VIDEO RAM
	output wire	O_WEN,
	output wire	[ADDR_DEPTH-1: 0] O_ADDR,
	output wire	[DATA_DEPTH-1: 0] O_DATA
);

///////////////////////////////////////////////////////////////////////////
// V base address
///////////////////////////////////////////////////////////////////////////
wire area_disp  = I_DISP;
wire area_blank = I_DISP;

reg de_polarity;

reg vsync_polarity;
reg [1:0] vsync_in;
reg de_in;

reg [ADDR_DEPTH-1:0] waddr;
reg [DATA_DEPTH-1:0] wdata;
reg wcycle;

//
reg [11:0] hcnt;
reg [11:0] vcnt;

// vsync detent signal
wire vsync_start = (vsync_in[1] == vsync_polarity) && (vsync_in[1] != vsync_in[0]);
wire vsync_detect = vsync_in[1] != vsync_polarity;
wire de           = I_DISP ^ de_polarity;

always @ (posedge I_CLK)
begin
	if(I_RESET)
	begin
		vsync_in       <= 2'b00;
		vsync_polarity <= 1'b0;
		de_polarity    <= 1'b0;
		waddr          <= 'd0;
		wdata          <= 'd0;
		wcycle         <= 'd0;
		hcnt           <= 'd0;
		vcnt           <= 'd0;
	end else begin
		// capture SYNC signals
		vsync_in <= { vsync_in[0],I_VSYNC};

		// auto detect DE polarity
		if(vsync_in[1]!=vsync_in[0])
			de_polarity    <= I_DISP;

		// setup videoram write enable & data
		wcycle  <= de;
		wdata   <= I_PIXEL;

		// auto detect VSYNC polarity
		if(de)
		begin
			// DISPLAY AREA
			vsync_polarity <= I_VSYNC;
			// count H-size
			hcnt <= hcnt +'d1;
		end else begin
			if(wcycle)// fall DE , end of H-draw
			begin
			     // update Hsize
				O_WIDTH  <= hcnt;
				hcnt     <= 'd0;
    			vcnt     <= vcnt +'d1;
			end
		end

		// vram write address
		if(vsync_start)
//		if(vsync_detect)
		begin
			// VSYNC return
			waddr        <= 'd0;
			wdata        <= 'd0;
			// update Vsize
			O_HEIGHT     <= vcnt;
			vcnt         <= 'd0;
			
		end else begin
			// after write pixel
			if(wcycle)
			begin
				waddr    <= waddr + 'd1;
			end
		end
	end
end
///////////////////////////////////////////////////////////////////////////
// output assign
///////////////////////////////////////////////////////////////////////////
assign O_ADDR = waddr;
assign O_DATA = wdata;
assign O_WEN  = wcycle;

endmodule
